Nt  =   2e3; % number of time steps (in dt units)
dt  =   .1  ; % time step, days
dt2 = .5*dt; % half a time step (for the Runge Kutta time stepping)
dt6 =  dt/6; % half a time step (for the Runge Kutta time stepping)

doPlots = 1;

X = zeros(5,Nt); % for evolving state vector

% grass growth functions, kg grass/acre/day
g1 = @(x) 17*x./(150 +x); % opportunistic,  fast ,   low  yielding
g2 = @(x) 20*x./(300 +x); % opportunistic, slower, higher yielding
g3 = @(x) 25./(1 + exp(-.02*(x-300))); % mature spcs, faster, lower  yield
g4 = @(x) 30./(1 + exp(-.02*(x-350))); % mature spcs, slower, higher yield

% grass consumption functions, kg grass/acre/day 
c1 = @(x,y) .01*y*(x./(500+x)); % less favorable
c2 = @(x,y) .02*y*(x./(500+x)); % more favorable
c3 = @(x,y) .03*y*(x./(500+x)); % less favorable
c4 = @(x,y) .04*y*(x./(500+x)); % more favorable

% grass competition functions, assume the more opportunistic the
% less fit, kg grass/acre/day 
m1 = @(x,y2,y3,y4) .0007*(y2*(x./(700+x)) + y3*(x./(600+x)) + y4*(x./(500+x)));
m2 = @(x,y1,y3,y4) .0006*(y1*(x./(800+x)) + y3*(x./(600+x)) + y4*(x./(500+x)));
m3 = @(x,y1,y2,y4) .0005*(y1*(x./(800+x)) + y2*(x./(700+x)) + y4*(x./(500+x)));
m4 = @(x,y1,y2,y3) .0004*(y1*(x./(800+x)) + y2*(x./(700+x)) + y3*(x./(600+x)));

% grass conversion efficiency functions for all 4 species. 
% Following Wilmshurst et al. (1995) Behavioral Ecology 6(2),
% 209-217, doi: 10.1093/beheco/6.2.209, we assume declining energy
% conversion efficiency with increased biomass, a proxy for grass age.
% The 2nd multiplicative factor is the cow to grass water content ratio
e = @(x) (.08+.04*exp(-x/500))*.8;

% combine all functions into a single dState/dt vecctor function.
% In the last line, cattle death rate, is 50% per year, or 0.5/365
F = @(x) [ g1(x(1)) - m1(x(1),x(2),x(3),x(4)) - c1(x(1),x(5)) ; ...
           g2(x(2)) - m1(x(2),x(1),x(3),x(4)) - c2(x(2),x(5)) ; ...
           g3(x(3)) - m1(x(3),x(1),x(2),x(4)) - c3(x(3),x(5)) ; ...
           g4(x(4)) - m1(x(4),x(1),x(2),x(3)) - c4(x(4),x(5)) ; ...
           e(x(1))*c1(x(1),x(5)) + ...
           e(x(2))*c2(x(2),x(5)) + ...
           e(x(3))*c3(x(3),x(5)) + ...
           e(x(4))*c4(x(4),x(5)) -  x(5)*0.0014 ];
           
% initial condition
X(1:4,1) =  50; % initial kg grass/acre
X(5,1)   = 2e3; % initial kgCattle/acre

if doPlots

  clf % plot growth functions
  gr = 1:1e3; % assumed resident grass mass, kg/acre
  plot(gr,g1(gr),'r',gr,g2(gr),'b',gr,g3(gr),'c',gr,g4(gr),'m','LineWidth',1)
  legend('grass 1','grass 2','grass 3','grass 4','Location','southeast')
  xlabel('resident grass, kg acre^{-1}')
  ylabel('grass growth, kg acre^{-1} day^{-1}')
  print -dpdf growthFunctions.pdf
  
  clf % plot consumption functions
  d = 0:50:1e3; % assumed kgGrazers/acre
  c = 0:1:14;   % contours
  subplot(221); contourf(gr,d,c1(gr,d'),c)
  xlabel('resident grass, kg acre^{-1}')
  ylabel('kg grazer acre^{-1}')
  caxis([0 14]); colorbar
  title('grass 1 consumption, kg acre^{-1} day^{-1}')
  subplot(222); contourf(gr,d,c2(gr,d'),c)
  xlabel('resident grass, kg acre^{-1}')
  ylabel('kg grazer acre^{-1}')
  caxis([0 14]); colorbar
  title('grass 2 consumption, kg acre^{-1} day^{-1}')
  subplot(223); contourf(gr,d,c3(gr,d'),c)
  xlabel('resident grass, kg acre^{-1}')
  ylabel('kg grazer acre^{-1}')
  caxis([0 14]); colorbar
  title('grass 3 consumption, kg acre^{-1} day^{-1}')
  subplot(224); contourf(gr,d,c4(gr,d'),c)
  xlabel('resident grass, kg acre^{-1}')
  ylabel('kg grazer acre^{-1}')
  caxis([0 14]); colorbar
  title('grass 4 consumption, kg acre^{-1} day^{-1}')
  print -dpdf consumptionFunctions.pdf

end

clf

for i = 2:Nt
  x  = X(:,i-1);
  k1 = F(x);
  k2 = F(x+dt2*k1);
  k3 = F(x+dt2*k2); 
  k4 = F(x+dt *k3);
  X(:,i) = X(:,i-1) + dt6*(k1+2*(k2+k3)+k4);
end